/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.calm.foundation.rcp.ui.internal.linking;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IStatus;

import com.ibm.team.calm.foundation.common.linking.OSLCResourceDescription.PickerType;
import com.ibm.team.calm.foundation.rcp.ui.linking.IItemPickerDialog;
import com.ibm.team.repository.common.transport.AbstractElementDescriptor;
import com.ibm.team.repository.common.transport.TeamServiceRegistryException;
import com.ibm.team.repository.common.util.NLS;

class ItemPickerDialogDescriptionElement extends AbstractElementDescriptor {
	
	protected ItemPickerDialogDescriptionElement(IConfigurationElement element) {
		super(element, ItemPickerDialogRegistry.ELEMENT_DIALOG_DESCRIPTION);
	}
	
	/* package */ void remove() throws TeamServiceRegistryException {
	}
	
	public ItemPickerDialogDescription getDialogDescription() {
		return new ItemPickerDialogDescription(ItemPickerDialogDescriptionElement.this);
	}
	
	public PickerType getPickerType() {
		String pickerType= getAttribute(ItemPickerDialogRegistry.ATTR_PICKER_TYPE).toLowerCase();
		if (ItemPickerDialogRegistry.VALUE_CREATION_PICKER.equals(pickerType)) {
			return PickerType.CREATION;
		}
		return PickerType.SELECTION;
	}
	
	public IItemPickerDialog getPickerDialog() throws TeamServiceRegistryException{
		return createItemPickerDialog();
	}
	
	private IItemPickerDialog createItemPickerDialog() throws TeamServiceRegistryException {
		IItemPickerDialog pickerDialog= null;

        String bundle= getBundleSymbolicName();
        String implementationClassName= getAttribute(ItemPickerDialogRegistry.ATTR_DIALOG_IMPLEMENTATION);

        try {
        	Object instance= createInstance(ItemPickerDialogRegistry.ATTR_DIALOG_IMPLEMENTATION);

        	if (instance == null) {
	            String message= NLS.bind(Messages.ItemPickerDialogDescriptionElement_FailedToCreateServiceInstance, bundle, implementationClassName);
	            throw new TeamServiceRegistryException(message);
	        }
        	if (instance instanceof IItemPickerDialog) {
	            pickerDialog= (IItemPickerDialog)instance;
        	} else {
        		String message= NLS.bind(Messages.ItemPickerDialogDescriptionElement_ServiceImplementedDoesNotImplementIService, implementationClassName, IItemPickerDialog.class.getName()); 
        		throw new TeamServiceRegistryException(message);
        	}
        } catch (CoreException ex) {
        	IStatus status= ex.getStatus();
        	TeamServiceRegistryException exp= new TeamServiceRegistryException(status.getMessage(), status.getException());
        	
            String message= NLS.bind(Messages.ItemPickerDialogDescriptionElement_FailedToCreateServiceInstance, bundle, implementationClassName);
            throw new TeamServiceRegistryException(message, exp);
        }

        return pickerDialog;
    }
}
