/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.calm.foundation.rcp.ui.internal.linking;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IConfigurationElement;

import com.ibm.team.calm.foundation.common.linking.OSLCResourceDescription.PickerType;
import com.ibm.team.calm.foundation.rcp.ui.internal.CALMFoundationRCPUIPlugin;
import com.ibm.team.repository.common.transport.TeamServiceRegistryException;
import com.ibm.team.repository.common.util.ExtensionRegistryReader;


public class ItemPickerDialogRegistry {
	static final String EXTENSION_POINT_ID= "itemPickerDialogs"; //$NON-NLS-1$
	
	static final String ELEMENT_DIALOG= "itemPickerDialog"; //$NON-NLS-1$
	static final String ELEMENT_DIALOG_DESCRIPTION= "dialogDescription"; //$NON-NLS-1$
	static final String ATTR_RESOURCE_ID= "resourceId"; //$NON-NLS-1$
	static final String ATTR_PICKER_TYPE= "pickerType"; //$NON-NLS-1$
	static final String ATTR_DIALOG_IMPLEMENTATION= "dialogImplementation"; //$NON-NLS-1$
	
	static final String VALUE_SELECTION_PICKER= "selection"; //$NON-NLS-1$
	static final String VALUE_CREATION_PICKER= "creation"; //$NON-NLS-1$

	private static ItemPickerDialogRegistry fgInstance;

	private Map<String, List<ItemPickerDialogDescription>> fItemToPickerDescriptionsMap;

	public static ItemPickerDialogDescription getItemPickerDescription(String resourceId, PickerType type) {
		List<ItemPickerDialogDescription> pickerDescriptions=  getInstance().getDialogs(resourceId);
		if (pickerDescriptions != null) {
			for (ItemPickerDialogDescription cur : pickerDescriptions) {
				if (type.equals(cur.getPickerType())) {
					return cur;
				}
			}
		}
		return null;
	}
	
	private static ItemPickerDialogRegistry getInstance() {
		if (fgInstance == null) {
			fgInstance= new ItemPickerDialogRegistry();
			fgInstance.initialize();
		}
		return fgInstance;
	}
	public List<ItemPickerDialogDescription> getDialogs(String resourceId) {
		return fItemToPickerDescriptionsMap.get(resourceId);
	}
	
	private ItemPickerDialogRegistry() {
		fItemToPickerDescriptionsMap= new HashMap<String, List<ItemPickerDialogDescription>>();	
	}
	
	private void initialize() {
		new ItemPickerDialogRegistryReader().start();
	}

	private class ItemPickerDialogRegistryReader extends ExtensionRegistryReader<ItemPickerDialogElement> {
		
		/*package*/ ItemPickerDialogRegistryReader() {
			super(CALMFoundationRCPUIPlugin.PLUGIN_ID, EXTENSION_POINT_ID);
		}
			
		@Override
		protected ItemPickerDialogElement handleExtensionAdded(IConfigurationElement element) throws TeamServiceRegistryException {
			ItemPickerDialogElement descriptor= new ItemPickerDialogElement(element);
			fItemToPickerDescriptionsMap.put(descriptor.getResourceId(), descriptor.getDialogDescriptions());
			return descriptor;
		}
		
		@Override
		protected void handleExtensionRemoved(IConfigurationElement element, ItemPickerDialogElement descriptor) throws TeamServiceRegistryException {
			descriptor.remove();
		}
	}
}