/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.calm.foundation.rcp.ui.internal.preview;

import java.net.URI;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.ibm.team.calm.foundation.client.internal.ResourceReference;
import com.ibm.team.calm.foundation.common.preview.ResourcePreview;
import com.ibm.team.foundation.common.URIReference;
import com.ibm.team.foundation.rcp.core.hyperlinks.ContextProvider;
import com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler;
import com.ibm.team.foundation.rcp.core.internal.HyperlinkHandlerRegistry;
import com.ibm.team.jface.BrowserLocationListener;
import com.ibm.team.repository.common.TeamRepositoryException;


/**
 * A {@link HyperlinkHandler} who's only purpose is to allow the
 * {@link BrowserLocationListener} to load specific URLs in the Browser and not
 * open it externally. This is important to display {@link ResourceReference}
 * that resolve with {@link ResourcePreview}.
 * 
 * @since 3.0
 */
public class ResourceReferenceHyperlinkHandler extends HyperlinkHandler {

	/**
	 * A makrer for URLs to indicate that the URL is from a
	 * {@link ResourcePreview}
	 */
	public static final String URL_MARKER= "#resourcedetailsmarker"; //$NON-NLS-1$

	/* Used by the Remote Login Dialog */
	//private static final String AUTH_MARKER_1= "blank.html?"; //$NON-NLS-1$
	//private static final String AUTH_MARKER_2= "blank.html?passthrough"; //$NON-NLS-1$

	private static final String[] MARKERS= new String[] { URL_MARKER /*, AUTH_MARKER_1, AUTH_MARKER_2 */};

	/*
	 * @see com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler#handles(java.net.URI)
	 */
	@Override
	public boolean handles(URI uri) {
		if (uri != null && ("http".equals(uri.getScheme()) || "https".equals(uri.getScheme()))) { //$NON-NLS-1$//$NON-NLS-2$
			String uriVal= uri.toString();
			for (String marker : MARKERS) {
				if (uriVal.endsWith(marker))
					return true;
			}
		}

		return false;
	}

	/*
	 * @see com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler#shouldOpen(java.net.URI)
	 */
	public boolean shouldOpen(URI uri) {
		return false; //Important to Load ResourcePreview in the Browser 
	}

	// ######################################################################################## //
	// ######### The Following Methods all Delegate to the Default Hyperlink Handler ########## //
	// ######################################################################################## //

	/*
	 * @see com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler#createHyperlink(java.lang.Object, org.eclipse.core.runtime.IProgressMonitor)
	 */
	@SuppressWarnings("restriction")
	@Override
	public URIReference createHyperlink(Object object, IProgressMonitor monitor) {
		return HyperlinkHandlerRegistry.getDefault().getDefaultHandler().createHyperlink(object, monitor);
	}

	/*
	 * @see com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler#createHyperlink(java.lang.Object)
	 */
	@SuppressWarnings( { "restriction", "deprecation" })
	@Override
	public URIReference createHyperlink(Object object) {
		return HyperlinkHandlerRegistry.getDefault().getDefaultHandler().createHyperlink(object);
	}

	/*
	 * @see com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler#links(java.lang.Object)
	 */
	@SuppressWarnings("restriction")
	@Override
	public boolean links(Object object) {
		return HyperlinkHandlerRegistry.getDefault().getDefaultHandler().links(object);
	}

	/*
	 * @see com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler#open(java.util.List, com.ibm.team.foundation.rcp.core.hyperlinks.ContextProvider, org.eclipse.core.runtime.IProgressMonitor)
	 */
	@SuppressWarnings("restriction")
	@Override
	public IStatus open(List<URI> uris, ContextProvider provider, IProgressMonitor monitor) {
		if (uris != null) {
			return HyperlinkHandlerRegistry.getDefault().getDefaultHandler().open(uris, provider, monitor);
		}
		return Status.OK_STATUS;
	}

	/*
	 * @see com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler#open(java.net.URI, com.ibm.team.foundation.rcp.core.hyperlinks.ContextProvider, org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void open(URI uri, ContextProvider provider, IProgressMonitor monitor) {
		if (uri != null) {
			HyperlinkHandlerRegistry.getDefault().getDefaultHandler().open(uri, provider, monitor);
		}
	}

	/*
	 * @see com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler#open(java.net.URI, com.ibm.team.foundation.rcp.core.hyperlinks.ContextProvider)
	 */
	@SuppressWarnings( { "deprecation", "restriction" })
	@Override
	public void open(URI uri, ContextProvider provider) {
		if (uri != null) {
			HyperlinkHandlerRegistry.getDefault().getDefaultHandler().open(uri, provider);
		}
	}

	/*
	 * @see com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler#open2(java.net.URI, com.ibm.team.foundation.rcp.core.hyperlinks.ContextProvider, org.eclipse.core.runtime.IProgressMonitor)
	 */
	@SuppressWarnings("restriction")
	@Override
	public IStatus open2(URI uri, ContextProvider provider, IProgressMonitor monitor) {
		if (uri != null) {
			return HyperlinkHandlerRegistry.getDefault().getDefaultHandler().open2(uri, provider, monitor);
		}
		return Status.OK_STATUS;
	}

	/*
	 * @see com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler#resolve(java.net.URI, com.ibm.team.foundation.rcp.core.hyperlinks.ContextProvider, org.eclipse.core.runtime.IProgressMonitor)
	 */
	@SuppressWarnings("restriction")
	@Override
	public Object resolve(URI uri, ContextProvider provider, IProgressMonitor monitor) throws CoreException {
		return HyperlinkHandlerRegistry.getDefault().getDefaultHandler().resolve(uri, provider, monitor);
	}

	/*
	 * @see com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler#resolve(java.net.URI, com.ibm.team.foundation.rcp.core.hyperlinks.ContextProvider)
	 */
	@SuppressWarnings( { "restriction", "deprecation" })
	@Override
	public Object resolve(URI uri, ContextProvider provider) throws TeamRepositoryException {
		return HyperlinkHandlerRegistry.getDefault().getDefaultHandler().resolve(uri, provider);
	}

	/*
	 * @see com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler#resolves(java.net.URI)
	 */
	@SuppressWarnings("restriction")
	@Override
	public boolean resolves(URI uri) {
		return HyperlinkHandlerRegistry.getDefault().getDefaultHandler().resolves(uri);
	}
}