/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.calm.foundation.rcp.ui.linking;

import java.util.List;

import com.ibm.team.calm.foundation.rcp.ui.internal.linking.Messages;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;


public class CALMBackLinkProblemDialog extends MessageDialog {
	
	public enum SaveStrategy {
		RETRY, IGNORE_BACKLINKS, REVERT_LINKS
	}
	
	private CALMBackLinkProblemDialog.SaveStrategy fResult;
	private final List<String> fBackLinkProblems;
	
	public CALMBackLinkProblemDialog(Shell shell, String title, List<String> backLinkProblems) {
		super(shell, title, null, Messages.CALMBackLinkProblemDialog_DIALOG_DESCRIPTION, MessageDialog.WARNING, new String[] { Messages.CALMBackLinkProblemDialog_BUTTON_SAVE, IDialogConstants.CANCEL_LABEL }, 1);
		fBackLinkProblems= backLinkProblems;
		fResult= SaveStrategy.RETRY;
		setShellStyle(getShellStyle() | SWT.RESIZE);
	}
	
	public CALMBackLinkProblemDialog.SaveStrategy getSaveStrategy() {
		return fResult;
	}
	
	@Override
	protected Control createCustomArea(Composite parent) {

		Composite composite= new Composite(parent, SWT.NONE);
		composite.setLayout(new GridLayout());
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		GridData layoutData;
		
		Text text= new Text(composite, SWT.V_SCROLL | SWT.BORDER | SWT.WRAP);
		String message= ""; //$NON-NLS-1$
		int count= 0;
		for(int i=0; i<fBackLinkProblems.size(); i++) {
			message+= fBackLinkProblems.get(i);
			if (i > 0) {
				message+= "\n\n"; //$NON-NLS-1$
			}
		}
		text.setText(message);
		layoutData= new GridData(SWT.FILL, SWT.FILL, true, true);
		layoutData.widthHint= convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
		if (count >= 8) {
			layoutData.heightHint= convertHeightInCharsToPixels(16);
		} else if (count < 8) {
			layoutData.heightHint= convertHeightInCharsToPixels(8);
		}
		text.setLayoutData(layoutData);
		
		Button button3= new Button(composite, SWT.RADIO);
		button3.setData(SaveStrategy.RETRY);
		button3.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		button3.setText(Messages.CALMBackLinkProblemDialog_RETRY_LABEL);
		button3.setSelection(fResult == button3.getData());
		button3.setFont(composite.getFont());
		
		Button button1= new Button(composite, SWT.RADIO);
		button1.setData(SaveStrategy.REVERT_LINKS);
		button1.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		button1.setText(Messages.CALMBackLinkProblemDialog_REVERT_LABEL);
		button1.setSelection(fResult == button1.getData());
		button1.setFont(composite.getFont());
		
		Button button2= new Button(composite, SWT.RADIO);
		button2.setData(SaveStrategy.IGNORE_BACKLINKS);
		button2.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		button2.setText(Messages.CALMBackLinkProblemDialog_SKIP_BACK_LINKS_LABEL);
		button2.setSelection(fResult == button2.getData());
		button2.setFont(composite.getFont());
		
		Composite warningComposite= new Composite(composite, SWT.NONE);
		GridLayout layout= new GridLayout();
		layout.numColumns= 2;
		layout.marginWidth= 0;
		layout.marginHeight= 0;
		warningComposite.setLayout(layout);
		layoutData= new GridData(GridData.FILL, GridData.CENTER, true, false);
		layoutData.horizontalIndent= convertWidthInCharsToPixels(2);
		warningComposite.setLayoutData(layoutData);
		warningComposite.setFont(composite.getFont());
		
		final Label imageLabel1= new Label(warningComposite, SWT.LEFT | SWT.WRAP);
		imageLabel1.setImage(Dialog.getImage(Dialog.DLG_IMG_MESSAGE_WARNING));
		imageLabel1.setLayoutData(new GridData(GridData.BEGINNING, GridData.CENTER, false, false, 1, 1));
		imageLabel1.setEnabled(fResult == SaveStrategy.IGNORE_BACKLINKS);
		
		final Label label= new Label(warningComposite, SWT.WRAP);
		layoutData= new GridData(SWT.FILL, SWT.CENTER, true, false);
		layoutData.widthHint= convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
		label.setLayoutData(layoutData);
		label.setText(Messages.CALMBackLinkProblemDialog_SKIP_BACK_LINKS_INFO);
		label.setEnabled(fResult == SaveStrategy.IGNORE_BACKLINKS);
		label.setFont(warningComposite.getFont());
		
		SelectionListener listener= new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (e.widget instanceof Button && ((Button) e.widget).getSelection()) {
					fResult= (CALMBackLinkProblemDialog.SaveStrategy) e.widget.getData();
				}
				imageLabel1.setEnabled(fResult == SaveStrategy.IGNORE_BACKLINKS);
				label.setEnabled(fResult == SaveStrategy.IGNORE_BACKLINKS);
			}
		};
		button1.addSelectionListener(listener);
		button2.addSelectionListener(listener);
		button3.addSelectionListener(listener);
		
		return composite;
	}
}