/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.calm.foundation.rcp.ui.linking;

import java.util.Collection;

import org.eclipse.jface.window.Window;
import org.eclipse.ui.IWorkbenchWindow;

import com.ibm.team.calm.foundation.common.linking.OSLCReference;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Interface for implementing picker dialogs for selecting items from a project area.
 * 
 * Note: Since 4.0.5, implementors of the <code>com.ibm.team.calm.foundation.rcp.ui.itemPickerDialogs</code> extension point
 * should extend from {@link com.ibm.team.calm.foundation.rcp.ui.linking.AbstractItemPickerDialog} as well. 
 * 
 * @since 3.0
 */
public interface IItemPickerDialog {

	/**
	 * Opens the picker dialog to select OSLC references
	 * 
	 * @param window -- parent window of the dialog
	 * @param project -- project area in which items can be selected
	 * @return -- error code indicating whether the dialog was canceled, see {@link Window} 
	 * @throws TeamRepositoryException
	 */
	public abstract int open(IProjectAreaHandle project, IWorkbenchWindow window) throws TeamRepositoryException;
	
	/**
	 * @return collection of selected OSLC references, or an empty collection 
	 */
	public abstract Collection<OSLCReference> getReferences();
}

