/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2006, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.calm.foundation.rcp.ui.linking;

import java.net.SocketTimeoutException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.wizard.ProgressMonitorPart;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.forms.IMessage;

import com.ibm.icu.text.Collator;
import com.ibm.team.calm.foundation.client.IServiceProviderClient;
import com.ibm.team.calm.foundation.common.ServiceProvider;
import com.ibm.team.calm.foundation.common.linking.CALMLinkTypeInformation;
import com.ibm.team.calm.foundation.common.linking.OSLCResourceDescription.PickerType;
import com.ibm.team.calm.foundation.common.linking.OSLCResourceDescription.ServiceDocument;
import com.ibm.team.calm.foundation.common.linking.PickerDescription;
import com.ibm.team.calm.foundation.rcp.ui.internal.CALMFoundationRCPUIPlugin;
import com.ibm.team.calm.foundation.rcp.ui.internal.HelpContextIds;
import com.ibm.team.calm.foundation.rcp.ui.internal.linking.ItemPickerDialogDescription;
import com.ibm.team.calm.foundation.rcp.ui.internal.linking.ItemPickerDialogRegistry;
import com.ibm.team.calm.foundation.rcp.ui.internal.linking.Messages;
import com.ibm.team.calm.foundation.rcp.ui.linking.CALMResourcePickerDialog.ServiceProviderDescriptor;
import com.ibm.team.foundation.rcp.ui.util.FoundationUIJob;
import com.ibm.team.jface.util.UIUpdaterJob;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.PermissionDeniedException;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.repository.transport.client.AuthenticationException;



class ServiceProviderSelectionDialog extends TitleAreaDialog {

	private final ServiceProviderDescriptor fServiceProviderDesc;
	private final Collection<ServiceProvider> fServiceProviders;
	private final ITeamRepository fRepository;

	private ListViewer fProviderList;
	private Button fSelectionButton;
	private Button fCreationButton;
	private ComboViewer fSelectionCombo;
	private ComboViewer fCreationCombo;
	private ProgressMonitorPart fMonitorPart;
	
	private IProjectAreaHandle fLocalProject;
	private ServiceProvider fCurrentProvider;
	
	private ItemPickerDialogDescription fLocalCreationPicker;
	private ItemPickerDialogDescription fLocalSelectionPicker;
	private List<PickerDescription> fRemoteCreationPickers;
	private List<PickerDescription> fRemoteSelectionPickers;
	private PickerDescription fCurrentRemoteCreationPicker;
	private PickerDescription fCurrentRemoteSelectionPicker;
	private final String fGlobalConfigurationParameter;

	private PickerType fPickerType;

	private final static Map<ServiceProviderDescriptor, ServiceProvider> fgServiceProviderCache= new HashMap<ServiceProviderDescriptor, ServiceProvider>();
	private final static Comparator<PickerDescription> fgPickerDescriptionComparator= new Comparator<PickerDescription>() {
		@Override
		public int compare(PickerDescription p1, PickerDescription p2) {
			String left= p1 != null ? p1.getTitle() : null;
			String right= p2 != null ? p2.getTitle() : null;
			if (left == null && right == null) {
				return 0;
			}
			if (left == null)
				return 1;
			if (right == null)
				return -1;
			return Collator.getInstance().compare(left, right);
		}
	};

	private final CALMLinkTypeInformation fLinkType;

	public ServiceProviderSelectionDialog(CALMLinkTypeInformation linkType, Collection<ServiceProvider> serviceProviders, ITeamRepository repository, Shell parentShell, ServiceProviderDescriptor serviceProviderDesc) {
		this(linkType, serviceProviders, repository, parentShell, serviceProviderDesc, null);
	}

	public ServiceProviderSelectionDialog(CALMLinkTypeInformation linkType, Collection<ServiceProvider> serviceProviders, ITeamRepository repository, Shell parentShell, ServiceProviderDescriptor serviceProviderDesc, String globalConfigurationParameter) {
		super(parentShell);
		setShellStyle(getShellStyle() | SWT.RESIZE);
		fLinkType= linkType;
		fServiceProviders= serviceProviders;
		fRepository= repository;
		fServiceProviderDesc= serviceProviderDesc;
		setHelpAvailable(true);
		fGlobalConfigurationParameter= globalConfigurationParameter;
	}
	
	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(Messages.ServiceProviderSelectionDialog_DIALOG_TITLE);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite= (Composite)super.createDialogArea(parent);
		initializeDialogUnits(composite);
		HelpContextIds.hookHelpListener(composite, HelpContextIds.ADD_CLM_ARTIFACT_LINK);
		setTitle(NLS.bind(Messages.ServiceProviderSelectionDialog_DIALOG_TITLE_ACTION, fLinkType.getItemLinkType().getTargetEndPointDescriptor().getDisplayName()));
		
		composite.setLayout(new GridLayout(1, false));
		GridData layoutData= new GridData(SWT.FILL, SWT.FILL, true, true);
		composite.setLayoutData(layoutData);
		
		Composite projComp= new Composite(composite, SWT.NONE);
		projComp.setLayout(new GridLayout(1, false));
		projComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		Label projectLabel= new Label(projComp, SWT.WRAP);
		GridData labelData = new GridData();
		labelData.widthHint = convertWidthInCharsToPixels(50);
		labelData.horizontalAlignment = SWT.FILL;	
		projectLabel.setLayoutData(labelData);
		projectLabel.setText(Messages.ServiceProviderSelectionDialog_LABEL_SERVICE_PROVIDER);
		createServiceProviderList(projComp);

		Composite actionComp= new Composite(composite, SWT.NONE);
		actionComp.setLayout(new GridLayout(2, false));
		actionComp.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));

		Label actionLabel= new Label(actionComp, SWT.WRAP);
		GridData actionData = new GridData();
		actionData.widthHint = convertWidthInCharsToPixels(50);
		actionData.horizontalSpan= 2;
		actionData.horizontalAlignment = SWT.FILL;	
		actionLabel.setLayoutData(actionData);
		actionLabel.setText(Messages.ServiceProviderSelectionDialog_LABEL_LINK_ACTION);

		Composite columComp= new Composite(actionComp, SWT.NONE);
		columComp.setLayout(new GridLayout(2, false));
		columComp.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
		((GridData)columComp.getLayoutData()).horizontalSpan= 2;
		
		fSelectionButton= new Button(columComp, SWT.RADIO);
		fSelectionButton.setLayoutData(new GridData(SWT.CENTER));
		((GridData)fSelectionButton.getLayoutData()).verticalAlignment= SWT.CENTER;
		fSelectionButton.setText(Messages.ServiceProviderSelectionDialog_LABEL_SELECTION_BUTTON);
		fSelectionButton.addSelectionListener( new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent evt) {
				fPickerType= PickerType.SELECTION;
			}
		});
		fSelectionButton.setEnabled(false);
		
		fSelectionCombo= new ComboViewer(columComp, SWT.READ_ONLY);
		fSelectionCombo.setContentProvider(new PickerContentProvider());
		fSelectionCombo.setLabelProvider(new PickerLabelProvider());
		fSelectionCombo.getCombo().setLayout(new GridLayout(1, false));
		fSelectionCombo.getCombo().setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
		((GridData)fSelectionCombo.getCombo().getLayoutData()).verticalAlignment= SWT.CENTER;
		fSelectionCombo.addSelectionChangedListener( new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent evt) {
				IStructuredSelection sel= (IStructuredSelection)evt.getSelection();
				if (!sel.isEmpty()) {
					fCurrentRemoteSelectionPicker= (PickerDescription)sel.getFirstElement();
					fSelectionButton.setSelection(true);
					fCreationButton.setSelection(false);
				}
			}
		});
		fSelectionCombo.getCombo().setEnabled(false);
		
		fCreationButton= new Button(columComp, SWT.RADIO);
		fCreationButton.setLayoutData(new GridData(SWT.CENTER));
		((GridData)fCreationButton.getLayoutData()).verticalAlignment= SWT.CENTER;
		fCreationButton.setText(Messages.ServiceProviderSelectionDialog_LABEL_CREATION_BUTTON);
		fCreationButton.addSelectionListener( new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent evt) {
				fPickerType= PickerType.CREATION;
			}
		});
		fCreationButton.setEnabled(false);
		
		fCreationCombo= new ComboViewer(columComp, SWT.READ_ONLY);
		fCreationCombo.setContentProvider(new PickerContentProvider());
		fCreationCombo.setLabelProvider(new PickerLabelProvider());
		fCreationCombo.getCombo().setLayout(new GridLayout(1, false));
		fCreationCombo.getCombo().setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
		((GridData)fCreationCombo.getCombo().getLayoutData()).verticalAlignment= SWT.CENTER;
		fCreationCombo.addSelectionChangedListener( new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent evt) {
				IStructuredSelection sel= (IStructuredSelection)evt.getSelection();
				if (!sel.isEmpty()) {
					fCurrentRemoteCreationPicker= (PickerDescription)sel.getFirstElement();
					fSelectionButton.setSelection(false);
					fCreationButton.setSelection(true);
				}
			}
		});
		fCreationCombo.getCombo().setEnabled(false);
		
		fMonitorPart= new BackgroundProgressMonitorPart(composite, new GridLayout(1, false));
		fMonitorPart.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		fMonitorPart.setVisible(false);

		applyDialogFont(composite);
		
		updateServiceProviderList();
		
		composite.pack();
		return composite;
	}
	
	@Override
	public void create() {
		super.create();
		updateOkButton(false);
	}
	
	public IProjectAreaHandle getLocalProject() {
		return fLocalProject;
	}
	
	public PickerDescription getOSLCPickerDescription() {
		if (PickerType.CREATION.equals(fPickerType)) {
			return fCurrentRemoteCreationPicker;
		} else {
			return fCurrentRemoteSelectionPicker;
		}
	}
	
	public ItemPickerDialogDescription getItemPickerDescription() {
		if (PickerType.CREATION.equals(fPickerType)) {
			return fLocalCreationPicker;
		} else {
			return fLocalSelectionPicker;
		}
	}

	public ServiceProvider getCALMLink() {
		return fCurrentProvider;
	}
	
	private void createServiceProviderList(Composite parent) {
		org.eclipse.swt.widgets.List list= new org.eclipse.swt.widgets.List(parent, SWT.BORDER | SWT.READ_ONLY | SWT.V_SCROLL);
		fProviderList= new ListViewer(list);
		fProviderList.setContentProvider(new SelectionDialogContentProvider());
		fProviderList.setLabelProvider(new ServiceProviderLabelProvider());
		GridData data= new GridData(SWT.FILL, SWT.FILL, true, true);
		data.heightHint= convertHeightInCharsToPixels(Math.max(4, Math.min(fServiceProviders.size(), 7))); // not more than 7 rows, but no less than 4
		data.horizontalIndent= 10;
		fProviderList.getList().setLayoutData(data);
		fProviderList.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				final ISelection selection= event.getSelection();
				if (selection != null 
						&& !selection.isEmpty() 
						&& selection instanceof IStructuredSelection) {
					Object selObj= ((IStructuredSelection)selection).getFirstElement();
					if (selObj instanceof ServiceProvider) {
						if (fCurrentProvider== null || !selObj.equals(fCurrentProvider)) {
							ServiceProvider selProvider= (ServiceProvider) selObj;
							updateButtons(selProvider);
						} else {
							updateOkButton(true);
						}
						if (fServiceProviderDesc != null) {
							fgServiceProviderCache.put(fServiceProviderDesc, fCurrentProvider);	
						}
					} else {
						updateOkButton(false);
					}
				}
			}
		});
	}
	
	private void updateButtons(ServiceProvider provider) {
		fCurrentProvider= provider;
		fSelectionButton.setEnabled(false);
		fCreationButton.setEnabled(false);
		updateStatus(Status.OK_STATUS);
		updateOkButton(false);
		
		showMonitor(true);
		fLocalProject= null;
		UIUpdaterJob updaterJob= new UIUpdaterJob("") { //$NON-NLS-1$
			private IStatus fStatus= Status.OK_STATUS;
			
			@Override
			public IStatus runInBackground(IProgressMonitor monitor) {
				try {
					IServiceProviderClient calmFoundationClient= (IServiceProviderClient)fRepository.getClientLibrary(IServiceProviderClient.class);
					SubMonitor subMon= SubMonitor.convert(fMonitorPart, Messages.ServiceProviderSelectionDialog_TASK_FIND_REPOSITORY_CONNECTION, 1);
					fLocalProject= calmFoundationClient.findRepositoryConnection(fCurrentProvider, subMon.newChild(1));
					if (fLocalProject != null) {
						String targetResourceId= fCurrentProvider.getLinkTypeInfo().getTargetResourceId();
						fLocalCreationPicker= ItemPickerDialogRegistry.getItemPickerDescription(targetResourceId, PickerType.CREATION);
						fLocalSelectionPicker= ItemPickerDialogRegistry.getItemPickerDescription(targetResourceId, PickerType.SELECTION);
					} else {
						fLocalCreationPicker= null;
						fLocalSelectionPicker= null;
					}
					
					ServiceDocument serviceDocument= null;
					if (fLocalCreationPicker == null) {
						subMon= SubMonitor.convert(fMonitorPart, Messages.ServiceProviderSelectionDialog_TASK_RETRIEVE_REMOTE_DIALOG_INFO, 1);
						serviceDocument= calmFoundationClient.fetchServiceDocument(fCurrentProvider, subMon.newChild(1));
						if (serviceDocument != null) {
							fRemoteCreationPickers= sortPickers(serviceDocument.getPickerDescriptions(PickerType.CREATION, fLinkType, fGlobalConfigurationParameter));
						}
					}
					if (fLocalSelectionPicker == null) {
						if (serviceDocument == null) {
							subMon= SubMonitor.convert(fMonitorPart, Messages.ServiceProviderSelectionDialog_TASK_RETRIEVE_REMOTE_DIALOG_INFO, 1);
							serviceDocument= calmFoundationClient.fetchServiceDocument(fCurrentProvider, subMon.newChild(1));
						} 
						if (serviceDocument != null) {
							fRemoteSelectionPickers= sortPickers(serviceDocument.getPickerDescriptions(PickerType.SELECTION, fLinkType, fGlobalConfigurationParameter));
						}
					}
				} catch (TeamRepositoryException ex) {
					fStatus= createUserLevelStatus(ex);
					CALMFoundationRCPUIPlugin.getDefault().log(fStatus);
				} finally {
					if (fMonitorPart != null && !fMonitorPart.isDisposed()) fMonitorPart.done();
				} 
				return Status.OK_STATUS;
			}
			
			private List<PickerDescription> sortPickers(List<PickerDescription> pickerDescriptions) {
				if (pickerDescriptions != null && pickerDescriptions.size() > 1) {
					List<PickerDescription> sorted= new ArrayList<PickerDescription>(pickerDescriptions.size());
					sorted.addAll(pickerDescriptions);
					Collections.sort(sorted, fgPickerDescriptionComparator);
					return sorted;
				}
				return pickerDescriptions;
			}

			@Override
			public IStatus runInUI(IProgressMonitor monitor) {
				showMonitor(false);
				if (!fSelectionButton.isDisposed() && !fCreationButton.isDisposed()) {
					fSelectionButton.setEnabled(fLocalSelectionPicker != null || (fRemoteSelectionPickers != null && !fRemoteSelectionPickers.isEmpty()));
					fCreationButton.setEnabled(fLocalCreationPicker != null || (fRemoteCreationPickers != null && !fRemoteCreationPickers.isEmpty()));
					if (fStatus.isOK() && !fSelectionButton.isEnabled() && !fCreationButton.isEnabled()) {
						fStatus= new Status(IStatus.INFO, CALMFoundationRCPUIPlugin.PLUGIN_ID, Messages.ServiceProviderSelectionDialog_INFO_UNSUPPORTED_ARTIFIACT);
					}
					if (fSelectionButton.isEnabled()) {
						fSelectionButton.setSelection(true);
						fCreationButton.setSelection(false);
						fPickerType= PickerType.SELECTION;
					} else if (fCreationButton.isEnabled()) {
						fSelectionButton.setSelection(false);
						fCreationButton.setSelection(true);
						fPickerType= PickerType.CREATION;
					}
					if (fLocalSelectionPicker != null) {
						fSelectionCombo.setInput(new Object[] { fLocalSelectionPicker });
						fSelectionCombo.getCombo().select(0);
						fSelectionCombo.getCombo().setEnabled(false);
					} else if (fRemoteSelectionPickers != null && !fRemoteSelectionPickers.isEmpty()) {
						Object[] input= fRemoteSelectionPickers.toArray();
						fSelectionCombo.setInput(input);
						if (input.length > 0) {
							fSelectionCombo.getCombo().select(0);
							fCurrentRemoteSelectionPicker= fRemoteSelectionPickers.get(0);
						}
						fSelectionCombo.getCombo().setEnabled(input.length > 1);
					}
					if (fLocalCreationPicker != null) {
						fCreationCombo.setInput(new Object[] { fLocalCreationPicker });
						fCreationCombo.getCombo().select(0);
						fCreationCombo.getCombo().setEnabled(false);
					} else if (fRemoteCreationPickers != null && !fRemoteCreationPickers.isEmpty()) {
						Object[] input= fRemoteCreationPickers.toArray();
						fCreationCombo.setInput(input);
						if (input.length > 0) {
							fCreationCombo.getCombo().select(0);
							fCurrentRemoteCreationPicker= fRemoteCreationPickers.get(0);
						}
						fCreationCombo.getCombo().setEnabled(input.length > 1);
					}
					updateStatus(fStatus);
					updateOkButton(fStatus.isOK());
				}
				return Status.OK_STATUS;
			}
		};
		if (fCurrentProvider != null) {
			updaterJob.schedule();
		}
	}
	
	private void updateStatus(IStatus status) {
		switch (status.getSeverity()) {
		case IStatus.INFO:
			setMessage(status.getMessage(), IMessage.INFORMATION);
			break;
		case IStatus.WARNING:
			setMessage(status.getMessage(), IMessage.WARNING);
			break;
		case IStatus.ERROR:
			setMessage(status.getMessage(), IMessage.ERROR);
			break;
		default:
			setMessage(Messages.ServiceProviderSelectionDialog_DIALOG_TITLE_MESSAGE);
			break;
		}
	}

	private void updateOkButton(boolean isOk) {
		Button okBtn= getButton(IDialogConstants.OK_ID);
		if (okBtn != null) {
			okBtn.setEnabled(isOk);
		}
	}

	private void updateServiceProviderList() {
		if (fServiceProviders != null && !fServiceProviders.isEmpty()) {
			ServiceProvider[] input= fServiceProviders.toArray(new ServiceProvider[fServiceProviders.size()]);
			ServiceProvider lastSelectedProvider= null;
			
			if (fServiceProviderDesc != null) {
				lastSelectedProvider= fgServiceProviderCache.get(fServiceProviderDesc);
			}
			
			fProviderList.setInput(input);
			if (input.length == 1) {
				fProviderList.getList().select(0);
				lastSelectedProvider= input[0];
			} else if (input.length > 0 && lastSelectedProvider != null) {
				String item= ((ILabelProvider)fProviderList.getLabelProvider()).getText(lastSelectedProvider);
				int    index= fProviderList.getList().indexOf(item);
				fProviderList.getList().select(index);
			}
			updateButtons(lastSelectedProvider);
		}
	}
	
	private Status createUserLevelStatus(Exception ex) {
		Throwable target= ex.getCause() == null ? ex : ex.getCause();

		String host= URI.create(fCurrentProvider.getProviderServiceUrl()).getHost();
		Status status= null;
		if (target instanceof AuthenticationException) {
			status= new Status(IStatus.WARNING, CALMFoundationRCPUIPlugin.PLUGIN_ID, NLS.bind(Messages.ServiceProviderSelectionDialog_WARNING_SERVICE_PROVIDER_ACCESS_DENIED, host), target);
		} else if (target instanceof SocketTimeoutException) {
			status= new Status(IStatus.WARNING, CALMFoundationRCPUIPlugin.PLUGIN_ID, NLS.bind(Messages.ServiceProviderSelectionDialog_WARNING_SERVICE_PROVIDER_CONNECTION_TIMEOUT, host), target);
		} else if (target instanceof PermissionDeniedException) {
			status= new Status(IStatus.ERROR, CALMFoundationRCPUIPlugin.PLUGIN_ID, NLS.bind(Messages.ServiceProviderSelectionDialog_ERROR_SERVICE_PROVIDER_PERMISSION_DENIED, host), target);
		} else {
			status= new Status(IStatus.ERROR, CALMFoundationRCPUIPlugin.PLUGIN_ID, NLS.bind(Messages.ServiceProviderSelectionDialog_ERROR_ACCESSING_SERVICE_DOCUMENT, host), target);
		}
		return status;
	}

	private void showMonitor(final boolean b) {
		if (fMonitorPart != null && !fMonitorPart.isDisposed())
			fMonitorPart.setVisible(b);
	}

	private class BackgroundProgressMonitorPart extends ProgressMonitorPart {
		private static final String NO_NAME= ""; //$NON-NLS-1$

		public BackgroundProgressMonitorPart(Composite parent, Layout layout) {
			super(parent, layout);
		}
		
		@Override
		public void beginTask(final String name, final int totalWork) {
			if (isDisposed()) return;
			new FoundationUIJob(NO_NAME) {
				@Override
				protected IStatus runProtectedInUI(IProgressMonitor monitor) throws Exception {
					BackgroundProgressMonitorPart.super.beginTask(name, totalWork);
					return Status.OK_STATUS;
				}
			}.schedule();
		}
		
		@Override
		public void internalWorked(final double work) {
			if (isDisposed()) return;
			new FoundationUIJob(NO_NAME) {
				@Override
				protected IStatus runProtectedInUI(IProgressMonitor monitor) throws Exception {
					BackgroundProgressMonitorPart.super.internalWorked(work);
					return Status.OK_STATUS;
				}
			}.schedule();
		}
		
		@Override
		public void done() {
			if (isDisposed()) return;
			new FoundationUIJob(NO_NAME) {
				@Override
				protected IStatus runProtectedInUI(IProgressMonitor monitor) throws Exception {
					BackgroundProgressMonitorPart.super.done();
					return Status.OK_STATUS;
				}
			}.schedule();
		}
		
		@Override
		protected void updateLabel() {
			if (isDisposed()) return;
			new FoundationUIJob(NO_NAME) {
				@Override
				protected IStatus runProtectedInUI(IProgressMonitor monitor) throws Exception {
					BackgroundProgressMonitorPart.super.updateLabel();
					return Status.OK_STATUS;
				}
			}.schedule();
		}
	}
	
	private static class ServiceProviderLabelProvider implements ILabelProvider {
		@Override
		public String getText(Object element) {
			if (element != null && element instanceof ServiceProvider) {
				return ((ServiceProvider)element).getProviderName();
			}
			return element != null ? element.toString() : null;
		}
		@Override
		public Image getImage(Object element) { return null; }
		@Override
		public boolean isLabelProperty(Object element, String property) { return false; }
		@Override
		public void dispose() {}
		@Override
		public void addListener(ILabelProviderListener listener) {}
		@Override
		public void removeListener(ILabelProviderListener listener) {}
	}

	private class SelectionDialogContentProvider implements IStructuredContentProvider {
		@Override
		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof Object[]) {
				return (Object[])inputElement;
			}
			return new Object[0];
		}
		@Override
		public void dispose() {}
		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
	}
	
	private static class PickerLabelProvider implements ILabelProvider {
		@Override
		public String getText(Object element) {
			if (element != null) {
				if (element instanceof PickerDescription) {
					return ((PickerDescription)element).getLabel();
				} else if (element instanceof ItemPickerDialogDescription) {
					return ((ItemPickerDialogDescription)element).getLabel();
				}
			}
			return element != null ? element.toString() : null;
		}
		@Override
		public Image getImage(Object element) { return null; }
		@Override
		public boolean isLabelProperty(Object element, String property) { return false; }
		@Override
		public void dispose() {}
		@Override
		public void addListener(ILabelProviderListener listener) {}
		@Override
		public void removeListener(ILabelProviderListener listener) {}
	}

	private class PickerContentProvider implements IStructuredContentProvider {
		@Override
		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof Object[]) {
				return (Object[])inputElement;
			}
			return new Object[0];
		}
		@Override
		public void dispose() {}
		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
	}
}